<?php
// $Id$

/**
 * @file
 * OM Maximenu Render
 *
 * @author: Daniel Honrade http://drupal.org/user/351112
 *
 */

/**
 * Implementation of hook_init().
 *
 */
function om_maximenu_init() {
  global $_om_maximenu_variable;
  global $_om_maximenu_block_classes;  
  
  drupal_add_css(OM_MAXIMENU_PATH . '/css/om_maximenu.css');
  drupal_add_js(OM_MAXIMENU_PATH . '/js/om_maximenu.js'); 
  
  // add om tools custom block classes    
  if (module_exists('om_tools')) {

    $om_tools_values = variable_get('om_tools', '');
    $om_blocks = isset($om_tools_values['block']) ? $om_tools_values['block']: array();
    
    if (!empty($om_blocks) && ($om_blocks['block_classes_switch'] == 1)) {
      $_om_maximenu_block_classes = array();
      foreach ($om_blocks as $block_key => $block_class) {
        $block  = explode('___', $block_key);
        if (isset($block[1])) { // filters switch/reset        
          $module = $block[1];
          $delta  = $block[2];
          $_om_maximenu_block_classes[$module][$delta] = $block_class;
        }
      }
    }
  }

  // all menus settings
  $_om_maximenu_variable  = variable_get('om_maximenu', array());

  $maximenu = $_om_maximenu_variable;
  
  if (!empty($maximenu)) {
    foreach ($maximenu as $menu_key => $menu_content) {
      $visibility_values = isset($menu_content['menu_visibility']) ? $menu_content['menu_visibility']: array('visibility' => 0, 'pages' => '');
      $visibility = om_maximenu_menu_visibility($visibility_values);
      if ($menu_content['output'] == 'block') {
         // effects won't load if disabled blocks          
         $status = !empty($menu_content['block_options']['region']) ? 1: 0;
      }
      else {
        $status = 1; // all non-blocks
      }
      if ($visibility && $status) {      
        $action        = isset($menu_content['action']) ? $menu_content['action']: 'hover';  
        $active        = isset($menu_content['active']) ? $menu_content['active']: 0;  
        $longmenu      = isset($menu_content['longmenu']) ? $menu_content['longmenu']: 0;  
        $displace      = isset($menu_content['displace']) ? $menu_content['displace']: 0;  
        $skin          = isset($menu_content['skin']) ? $menu_content['skin']: 'bubble';
        $animated_bg   = isset($menu_content['animated_bg']) ? 1: 0; 
        $animated_link = isset($menu_content['animated_link']) ? 1: 0; 
        $jiggle        = isset($menu_content['jiggle']) ? 1: 0;   
        $scroll        = isset($menu_content['scroll']) ? 1: 0; 
        $tabbed        = isset($menu_content['tabbed']) ? $menu_content['tabbed']: '';                                                     
        $dock          = isset($menu_content['float_options']['dock']) ? 1: 0;
        $maximenu_name = om_string_name($menu_content['title']); 

        // add action per OM Maximenu
        om_maximenu_action($action, $maximenu_name, $displace, $tabbed); 

        // add css per OM Maximenu       
        drupal_add_css(OM_MAXIMENU_PATH . '/skin/' . $skin . '/' . $skin . '.css');

        // add 'active-trail' to menu li when submenus are active
        if ($active) om_maximenu_active_js($maximenu_name, $animated_bg); 

        // scroll long menu if it won't fit the container
        if ($longmenu) om_maximenu_longmenu_js($maximenu_name); 
        
        // animated bg effect
        if ($animated_bg) om_maximenu_bg_slider($maximenu_name);

        // animated link effect
        if ($animated_link) om_maximenu_link_slider($maximenu_name);
                 
        // animated link jiggle
        if ($jiggle) om_maximenu_jiggle($maximenu_name);
        
        // scrolling float
        if ($scroll) om_maximenu_scroll($maximenu_name);
              
        // jquery fisheye dock effect 
        if ($dock) om_maximenu_dock($maximenu_name);
      }
    }
  }
}

 
/**
 * Load all OM Maximenu blocks
 * 
 * This will process all created OM Maximenu blocks
 *
 */
function om_maximenu_block_load($maximenu_delta = '') {
  global $_om_maximenu_variable;
  $maximenu = $_om_maximenu_variable;
    
  $maximenu_delta = explode('-', $maximenu_delta);
  // ex. $maximenu_delta = 'om-maximenu-1';
  $delta = $maximenu_delta[2];
    
  $out = array();
  // om delta always starts with 1
  if (isset($maximenu[$delta]) && ($delta != 0)) {
    if (!empty($maximenu) && ($maximenu[$delta]['output'] == 'block')) {
      $visibility_values = isset($maximenu[$delta]['menu_visibility']) ? $maximenu[$delta]['menu_visibility']: array('visibility' => 0, 'pages' => '');
      $visibility = om_maximenu_menu_visibility($visibility_values);
      if ($visibility) {  
        $out['title']   = check_plain($maximenu[$delta]['title']);
        $out['content'] = om_maximenu_links_render($maximenu[$delta], $delta);
      }
    }
  }
  return $out;  
}


/**
 * Load all OM Maximenu blocks
 * 
 * This will process the main menu and floating blocks
 *
 */
function om_maximenu_load() {
  global $_om_maximenu_variable;
  $maximenu = $_om_maximenu_variable;

  $out = array();
  if (!empty($maximenu)) {
    foreach ($maximenu as $menu_key => $menu_content) {
      $visibility_values = isset($menu_content['menu_visibility']) ? $menu_content['menu_visibility']: array('visibility' => 0, 'pages' => '');
      $visibility = om_maximenu_menu_visibility($visibility_values);
      if (($menu_content['output'] != 'block') && $visibility) {  
        $out[$menu_key]['title']   = $menu_content['title'];
        $out[$menu_key]['output']  = $menu_content['output'];
        $out[$menu_key]['content'] = om_maximenu_links_render($maximenu[$menu_key], $menu_key);    
      }
    }
  }
  return $out;  
}


/**
 * OM Maximenu links rendering engine
 *
 */
function om_maximenu_links_render($links = array(), $menu_key = 1) {
  // immediately returns nothing if no links
  if (!isset($links['links'])) return NULL;
  
  // tabbed content
  $tabbed = (isset($links['tabbed']) && ($links['tabbed'] <> '')) ? 1: 0; 
  
  if ($tabbed) {
    // tabbed links        
    if ($links['tabbed'] == 'accordion') {
      $out = om_maximenu_accordion_render($links, $menu_key);    
    }
    elseif ($links['tabbed'] == 'roundabout') {
      $out = om_maximenu_roundabout_render($links, $menu_key);    
    }    
    else {
      $out = om_maximenu_tabbed_render($links, $menu_key);
    }
  }
  else {
    // tabbed links        
    $out = om_maximenu_submenu_render($links, $menu_key);  
  }
              
  // access on editing
  if (user_access('administer OM Maximenu')) $out .= '<div class="om-maximenu-edit">' . l(t('Maxedit'), 'admin/settings/om-maximenu/' . $menu_key . '/edit', array('query' => drupal_get_destination())) . '</div>';    
  
  return om_maximenu_wrapper($links, $out);
}


/**
 * OM Maximenu submenu type
 *
 */
function om_maximenu_submenu_render($links = array(), $menu_key = 1) {

  // menu name
  $maximenu_name = om_string_name($links['title']); 
  
  // disable link when active
  $disabled = (isset($links['disabled']) && ($links['disabled'] == 1)) ? 1: 0; 
  
  $out = '';
  $out .= '<div class="om-menu-ul-wrapper">';      
  $out .= '<ul id="om-menu-' . $maximenu_name . '" class="om-menu">';  
  
  // sort by weight
  uasort($links['links'], 'om_sort_by_weight'); 
  
  // first, last classes  
  $count = 0;
  $total = count($links['links']);
    
  foreach ($links['links'] as $key => $content) {
  
    // user roles and permissions
    $permission = om_maximenu_link_visible($content['roles']);
    
    if (!empty($permission)) {    
      $count++;

      // php on title
      if (isset($content['php_option']) && ($content['php_option'] == 1)) {
        ob_start();
        $content['link_title'] = eval($content['link_title']);
        //$output = ob_get_contents();
        ob_end_clean(); 
      }
        
      // span id
      $span_id = (!empty($content['id'])) ? ' id="' . om_string_name($content['id']) . '"': '';        

      // span title
      $span_title = (!empty($content['description']) && ($content['description_option'] == 'hover')) ? 'title="' . check_plain($content['description']) . '"': ''; 
    
      // link content      
      $link_option = om_maximenu_link_option($content);
      
      // a link attributes
      $attributes = om_maximenu_link_attrib($content);
            
      $out .= '<li id="om-leaf-' . $links['code'] . '-' . $key . '" class="' . om_maximenu_link_classes($content, $permission, $count, $total) . '">';    

		  // link is disabled when it's on its page
      $uri = trim($_SERVER['REQUEST_URI']);
		  $path_relative = base_path() . $content['path'];

      // Drupal doesn't accept # as valid path on field input
      $options = array();
      $options['query']    = isset($content['path_query']) ? $content['path_query']: '';
      $options['fragment'] = isset($content['path_fragment']) ? $content['path_fragment']: '';        
			
			// disables link when active
			$disable_active_link = ($disabled) ? ($path_relative != $uri) ? 1: 0: 1;
			
      if (!empty($content['path']) && $disable_active_link) {
        $path = ($content['path'] == '<front>') ? '': $content['path'];
        // there are too many things inside this $link_options which a regular l() can't handle
        //$out .= l($content['link_title'], $content['path'], array('attributes' => $attributes));
        $out .= '<a ' . drupal_attributes($attributes) . ' href="' . url($path, $options) . '">' . $link_option . '</a>';
      }
      elseif (!empty($options['query']) || !empty($options['fragment'])) {
        // base path is not necessary when path is empty but anchor is not or query, 
        $query    = !empty($options['query']) ? '?' . $options['query']: '';
        $fragment = !empty($options['fragment']) ? '#' . $options['fragment']: '';
        $out .= '<a ' . drupal_attributes($attributes) . ' href="' .  $query . $fragment . '">' . $link_option . '</a>';
      }      
      else {
        // useful when you just want a button for getting the content to show and not actually linking to anything
        $out .= '<span' . $span_id . ' class="' . $attributes['class'] . '" ' . $span_title . '>' . $link_option . '</span>';
      }
      
      // finally, getting the content
      $out .= om_maximenu_submenu_content_render($content['content']); 
         
      $out .= '</li>';
            
    } // permission closing
  } // foreach closing
  $out .= '</ul>';  
  $out .= '</div>'; // om-menu-ul-wrapper        
  return $out;
}


/**
 * OM Maximenu submenu content rendering engine
 *
 */
function om_maximenu_submenu_content_render($content = array()) {
  $out = '';
  if (!empty($content)) {
    $out .= '<div class="om-maximenu-content om-maximenu-content-nofade closed">';
    $out .= '<div class="om-maximenu-top"><div class="om-maximenu-top-left"></div><div class="om-maximenu-top-right"></div></div>';
    $out .= '<div class="om-maximenu-middle"><div class="om-maximenu-middle-left"><div class="om-maximenu-middle-right">';      
    $out .= om_maximenu_content_render($content);
    $out .= '<div class="om-clearfix"></div></div></div></div>';      
    $out .= '<div class="om-maximenu-bottom"><div class="om-maximenu-bottom-left"></div><div class="om-maximenu-bottom-right"></div></div>';    
    $out .= '<div class="om-maximenu-arrow"></div>';  
    $out .= '<div class="om-maximenu-open"><input type="checkbox" value="" />' . t('Stay') . '</div>';          
    $out .= '</div>'; 
  } 
  return $out;
}


/**
 * OM Maximenu accordion type
 *
 */
function om_maximenu_roundabout_render($links = array(), $menu_key = 1) {

  // menu name
  $maximenu_name = om_string_name($links['title']); 
        
  $out = '<ul id="om-menu-' . $maximenu_name . '">';  
  
  // sort by weight
  uasort($links['links'], 'om_sort_by_weight'); 
  
  // first, last classes  
  $count = 0;
  $total = count($links['links']);
    
  foreach ($links['links'] as $key => $content) {
  
    // user roles and permissions
    $permission = om_maximenu_link_visible($content['roles']);
    
    if (!empty($permission)) {    
      $count++;

      // php on title
      if (isset($content['php_option']) && ($content['php_option'] == 1)) {
        ob_start();
        $content['link_title'] = eval($content['link_title']);
        //$output = ob_get_contents();
        ob_end_clean(); 
      }
        
      // span id
      $span_id = (!empty($content['id'])) ? ' id="' . om_string_name($content['id']) . '"': '';        

      // span title
      $span_title = (!empty($content['description']) && ($content['description_option'] == 'hover')) ? 'title="' . check_plain($content['description']) . '"': ''; 
    
      // link content      
      $link_option = om_maximenu_link_option($content);
      
      // a link attributes
      $attributes = om_maximenu_link_attrib($content);
            
      $out .= '<li id="om-leaf-' . $links['code'] . '-' . $key . '" class="' . om_maximenu_link_classes($content, $permission, $count, $total) . '">';    
      
      // tabbed style doesn't need any a tag
      $out .= '<span' . $span_id . ' class="' . $attributes['class'] . '" ' . $span_title . '>' . $link_option . '</span>';
      
      // roundabout content
      $out .= om_maximenu_roundabout_content_render($content['content']); 
      
      $out .= '</li>';
            
    } // permission closing
  } // foreach closing
  $out .= '</ul>';  

  return $out;
}


/**
 * OM Maximenu roundabout content rendering engine
 *
 */
function om_maximenu_roundabout_content_render($content = array()) {
  $out = '';
  if (!empty($content)) {
    $out .= '<div class="om-maximenu-roundabout-content">';
    $out .= om_maximenu_content_render($content);
    $out .= '<div class="om-clearfix"></div>';      
    $out .= '<div class="om-maximenu-arrow"></div>';
    $out .= '</div>'; 
  } 
  return $out;  
}


/**
 * OM Maximenu accordion type
 *
 */
function om_maximenu_accordion_render($links = array(), $menu_key = 1) {

  // menu name
  $maximenu_name = om_string_name($links['title']); 
        
  $out = '<dl id="om-menu-' . $maximenu_name . '" class="easy-accordion">';  
  
  // sort by weight
  uasort($links['links'], 'om_sort_by_weight'); 
  
  // first, last classes  
  $count = 0;
  $total = count($links['links']);
    
  foreach ($links['links'] as $key => $content) {
  
    // user roles and permissions
    $permission = om_maximenu_link_visible($content['roles']);
    
    if (!empty($permission)) {    
      $count++;

      // php on title
      if (isset($content['php_option']) && ($content['php_option'] == 1)) {
        ob_start();
        $content['link_title'] = eval($content['link_title']);
        //$output = ob_get_contents();
        ob_end_clean(); 
      }
        
      // span id
      $span_id = (!empty($content['id'])) ? ' id="' . om_string_name($content['id']) . '"': '';        

      // span title
      $span_title = (!empty($content['description']) && ($content['description_option'] == 'hover')) ? 'title="' . check_plain($content['description']) . '"': ''; 
    
      // link content      
      $link_option = om_maximenu_link_option($content);
      
      // a link attributes
      $attributes = om_maximenu_link_attrib($content);
            
      //$out .= '<li id="om-leaf-' . $links['code'] . '-' . $key . '" class="' . om_maximenu_link_classes($content, $permission, $count, $total) . '">';    
      
      // tabbed style doesn't need any a tag
      $out .= '<dt' . $span_id . ' class="' . $attributes['class'] . '" ' . $span_title . '>' . $link_option . '</dt>';
      
      // tabbed content
      $out .= '<dd>' . om_maximenu_accordion_content_render($content['content']) . '</dd>'; 
            
    } // permission closing
  } // foreach closing
  $out .= '</dl>';  

  return $out;
}


/**
 * OM Maximenu accordion content rendering engine
 *
 */
function om_maximenu_accordion_content_render($content = array()) {
  $out = '';  
  if (!empty($content)) {
    $out .= '<div class="om-maximenu-accordion-content">';
    $out .= om_maximenu_content_render($content);
    $out .= '<div class="om-clearfix"></div>';      
    $out .= '<div class="om-maximenu-arrow"></div>';
    $out .= '</div>'; 
  } 
  return $out;  
}


/**
 * OM Maximenu tabbed type
 *
 */
function om_maximenu_tabbed_render($links = array(), $menu_key = 1) {

  // menu name
  $maximenu_name = om_string_name($links['title']); 

  $out = '';
  $out .= '<div class="om-menu-ul-wrapper">';        
  $out .= '<ul id="om-menu-' . $maximenu_name . '" class="om-menu">';  
  
  // sort by weight
  uasort($links['links'], 'om_sort_by_weight'); 
  
  // first, last classes  
  $count = 0;
  $total = count($links['links']);
    
  foreach ($links['links'] as $key => $content) {
  
    // user roles and permissions
    $permission = om_maximenu_link_visible($content['roles']);
    
    if (!empty($permission)) {    
      $count++;

      // php on title
      if (isset($content['php_option']) && ($content['php_option'] == 1)) {
        ob_start();
        $content['link_title'] = eval($content['link_title']);
        //$output = ob_get_contents();
        ob_end_clean(); 
      }
        
      // span id
      $span_id = (!empty($content['id'])) ? ' id="' . om_string_name($content['id']) . '"': '';        

      // span title
      $span_title = (!empty($content['description']) && ($content['description_option'] == 'hover')) ? 'title="' . check_plain($content['description']) . '"': ''; 
    
      // link content      
      $link_option = om_maximenu_link_option($content);
      
      // a link attributes
      $attributes = om_maximenu_link_attrib($content);
            
      $out .= '<li id="om-leaf-' . $links['code'] . '-' . $key . '" class="' . om_maximenu_link_classes($content, $permission, $count, $total) . '">';    

      // tabbed style doesn't need any a tag
      $out .= '<span' . $span_id . ' class="' . $attributes['class'] . '" ' . $span_title . '>' . $link_option . '</span>';
               
      $out .= '</li>';
            
    } // permission closing
  } // foreach closing
  $out .= '</ul>';  
  $out .= '</div>'; // om-menu-ul-wrapper        
  // tabbed content
  $out .= om_maximenu_tabbed_content_render($links); 

  return $out;
}


/**
 * OM Maximenu tabbed content rendering engine
 *
 */
function om_maximenu_tabbed_content_render($links = array()) {

  // tabbed contents
  $out = '<div class="om-maximenu-tabbed-content"><div class="om-maximenu-tabbed-content-inner">';

  foreach ($links['links'] as $key => $content) {

    // user roles and permissions
    $permission = om_maximenu_link_visible($content['roles']);
    
    if (!empty($permission) && !empty($content['content'])) {  
      $out .= '<div id="om-tabbed-content-' . $links['code'] . '-' . $key . '" class="om-tabbed-content om-tabbed-content-hide"><div class="om-tabbed-content-inner">'; 
      $out .= om_maximenu_content_render($content['content']);
      $out .= '<div class="om-clearfix"></div>';
      $out .= '</div></div>';        
    } 
  }
  $out .= '<div class="om-clearfix"></div>';
  $out .= '</div></div>'; 
  return $out;  
}


/**
 * OM Maximenu content rendering engine
 *
 */
function om_maximenu_content_render($content = array()) {
  global $_om_maximenu_block_classes;
  $block_classes = !empty($_om_maximenu_block_classes) ? $_om_maximenu_block_classes: array();  
  
  $count = 0;
  uasort($content, 'om_sort_by_weight'); 
  $total = count($content); 
  $out = '';
  foreach ($content as $key => $prop) {
    $count++;
        
    $module              = $prop['module'];
    $delta               = $prop['delta'];
    $title               = stripslashes(trim($prop['title'])); // allows markups
    $path                = ($prop['title_path'] == '<front>') ? '': $prop['title_path'];
    $options             = array();
    $options['query']    = isset($prop['title_path_query']) ? $prop['title_path_query']: '';
    $options['fragment'] = isset($prop['title_path_fragment']) ? $prop['title_path_fragment']: '';        
    $block_title         = (!empty($prop['title_path'])) ? '<a href="' . url($path, $options) .'" title="' . $title . '">' . $title . '</a>': $title;      
    $block               = module_invoke($module, 'block', 'view', $delta); 
    $classes = ($count == 1) ? ' first': '';
    if ($count == $total) $classes .= ' last'; 
    $classes .=  isset($block_classes[$module][$delta]) ? ' ' . $block_classes[$module][$delta]: ''; 
               
    $out .= '<div class="block block-' . $module . ' block-' . $module . '-id-' . $delta . $classes . '">';            
    $out .= ($title != '<none>') ? '<h3 class="title">' . $block_title . '</h3>': ''; 
    $out .= '<div class="content">' . $block['content'] . '</div>'; 
    if (user_access('administer blocks')) $out .= om_maximenu_block_edit($module, $delta, $title);
    $out .= '</div>';            
  }
  return $out;
}


/**
 * OM Maximenu wrapper
 *
 */
function om_maximenu_wrapper($links = array(), $content = NULL) {
  $out = '';
  if (!empty($content)) {
    global $user;  
    $maximenu_name = om_string_name($links['title']); 
    $code     = 'om-u'. $user->uid . '-' . mt_rand();
    $link_code = isset($links['code']) ? trim($links['code']): $code;    
    $out .= '<div id="om-maximenu-' . $maximenu_name . '" class="om-maximenu' . om_maximenu_classes($links) . ' code-' . $link_code . '"' . om_maximenu_inline_style($links) . '>';      
    //$out .= '<div id="om-maximenu-' . $maximenu_name . '-outer" class="om-maximenu-wrapper-outer">';
    $out .= ($links['output'] == 'float') ? '<div id="om-maximenu-' . $maximenu_name . '-inner" class="om-maximenu-wrapper-inner">' . $content . '</div>': $content;
    //$out .= '</div>';
    $out .= '</div>';    
  }
  return $out;
}


